"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildWebApp = void 0;
const webpack_1 = require("../service/builder/webpack");
const builder_1 = require("../service/builder");
const weapps_core_1 = require("../../weapps-core");
const generate_1 = require("./generate");
const material_1 = require("./material");
const copy_1 = require("./copy");
const prepare_1 = require("./prepare");
const webpack_2 = require("./webpack");
const index_1 = require("../mp/index");
const path_1 = __importDefault(require("path"));
const types_1 = require("../../types");
const mixMode_1 = require("../mp/mixMode");
const chalk_1 = __importDefault(require("chalk"));
function buildWebApp({ mainAppSerializeData, subAppSerializeDataList = [], dependencies, appKey = 'test', nodeModulesPath, runtime = types_1.RUNTIME.NONE, ignoreInstall = false, buildTypeList = ["web"], mode = "production", deployMode = types_1.DEPLOY_MODE.PREVIEW, watch = false, generateMpType = "app", generateMpPath = '', isCleanDistDir = false, plugins = [], extraData = {
    isComposite: false,
    compProps: {},
}, isCrossAccount = false, resourceAppid = undefined, domain = '', }, cb) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        if (!mainAppSerializeData) {
            console.error('无效的应用配置');
            return;
        }
        console.log('domain', domain);
        console.log('应用名', appKey);
        console.log('生成模式', generateMpType);
        if (generateMpType === "subpackage") {
            console.log('主包项目路径', generateMpPath);
        }
        let { appBuildDir, materialsDir } = runtime === types_1.RUNTIME.CI ? builder_1.getCompileDirs('app') : builder_1.getCompileDirs(appKey);
        const startTime = Date.now();
        if (buildTypeList.includes("mp")) {
            appBuildDir = path_1.default.join(appBuildDir, 'mp');
            const isMixMode = generateMpType === "subpackage";
            const apps = [mainAppSerializeData, ...subAppSerializeDataList];
            try {
                const result = yield index_1.generateWxMp({
                    weapps: apps,
                    projDir: appBuildDir,
                    appId: appKey,
                    domain: domain,
                    materials: dependencies,
                    plugins,
                    isProduction: mode === "production",
                    deployMode,
                    extraData,
                    isMixMode,
                    options: { isCrossAccount: !!isCrossAccount, resourceAppid },
                    buildTypeList,
                });
                if (isMixMode) {
                    generateMpPath = appBuildDir;
                    console.log(chalk_1.default.green('【混合模式】'), generateMpPath);
                    yield mixMode_1.handleMixMode({
                        apps,
                        generateMpPath,
                        miniprogramRoot: result.miniprogramRoot,
                        plugins,
                    });
                }
                cb === null || cb === void 0 ? void 0 : cb(null, Object.assign(Object.assign({}, result), { outDir: isMixMode && generateMpPath ? generateMpPath : appBuildDir, timeElapsed: Date.now() - startTime }));
                return appBuildDir;
            }
            catch (e) {
                console.log('generateWxMp error', e);
                cb && cb(e);
                return;
            }
        }
        else {
            appBuildDir = path_1.default.join(appBuildDir, 'h5');
            const mainAppData = weapps_core_1.deserialize(mainAppSerializeData);
            const subAppDataList = subAppSerializeDataList.map((sub) => weapps_core_1.deserialize(sub));
            const buildContext = {
                projDir: appBuildDir,
                appId: appKey,
                isProduction: mode === "production",
                materialLibs: dependencies,
                isMixMode: false,
                domain,
                mainAppData: mainAppSerializeData,
            };
            const { allAppUsedComps } = index_1.handleUsedComponents({
                buildContext,
                weapps: [mainAppSerializeData, ...subAppSerializeDataList],
                materials: dependencies,
            });
            const { publicPath, basename, assets = '' } = ((_a = mainAppData.appConfig) === null || _a === void 0 ? void 0 : _a.window) || {};
            const projectConfig = yield prepare_1.runPrepare(buildTypeList, appBuildDir, isCleanDistDir);
            const mpConfig = yield webpack_1.extractAndRemoveKbConfig(mainAppData, subAppDataList, appBuildDir);
            const jsAssets = assets.split(',').map((item) => { var _a; return (_a = (item || '')) === null || _a === void 0 ? void 0 : _a.trim(); });
            yield copy_1.runCopy(appBuildDir, mainAppData, {
                adminPortalKey: deployMode === types_1.DEPLOY_MODE.PREVIEW ? `${appKey}-preview` : appKey,
            });
            const runHandleMaterialTag = '======= buildWebApp-runHandleMaterial';
            console.time(runHandleMaterialTag);
            const filteredDependiences = dependencies.filter((lib) => {
                if (!allAppUsedComps[lib.name]) {
                    return false;
                }
                else {
                    return true;
                }
            });
            yield material_1.runHandleMaterial(appBuildDir, filteredDependiences, materialsDir, runtime, ignoreInstall);
            console.timeEnd(runHandleMaterialTag);
            yield generate_1.runGenerateCore(appBuildDir, mainAppData, subAppDataList, filteredDependiences, appKey, basename, buildTypeList, deployMode, runtime, ignoreInstall, extraData, domain);
            const doneCallback = prepare_1.createDoneCallBack({ appBuildDir, projectConfig }, cb);
            yield webpack_2.runWebpackCore({
                appBuildDir,
                mainAppData,
                subAppDataList,
                materialsDir,
                dependencies: filteredDependiences,
                nodeModulesPath,
                publicPath,
                mode,
                watch,
                appKey,
                cb: doneCallback,
                mpConfig,
                buildTypeList,
                generateMpType,
                generateMpPath,
                plugins,
                assets: jsAssets,
                domain,
            });
            return appBuildDir;
        }
    });
}
exports.buildWebApp = buildWebApp;
exports.default = buildWebApp;
