"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleUsedComponents = exports.writeLowCodeFiles = exports.generateWxMp = void 0;
const chalk_1 = __importDefault(require("chalk"));
const path_1 = __importDefault(require("path"));
const util_1 = require("util");
const fs = __importStar(require("fs-extra"));
const weapps_core_1 = require("../../weapps-core");
const config_1 = require("../config");
const mp_1 = require("../config/mp");
const generateFiles_1 = __importStar(require("../util/generateFiles"));
const materials_1 = require("./materials");
const webpack_1 = require("../service/builder/webpack");
const util_2 = require("./util");
const wxml_1 = require("./wxml");
const lowcode_1 = require("./lowcode");
const mp_config_1 = require("./mp_config");
const plugin_1 = require("../service/builder/plugin");
const plugin_2 = require("../core/plugin");
const util_3 = require("../util");
const style_1 = require("../util/style");
const dataSource_1 = require("../../utils/dataSource");
const types_1 = require("../../types");
const common_1 = require("../types/common");
const lodash_1 = require("lodash");
const junk = __importStar(require("../util/junk"));
const path_2 = __importDefault(require("path"));
const net_1 = require("../util/net");
const templateDir = config_1.appTemplateDir + '/mp/';
const em = chalk_1.default.blue.bold;
const error = chalk_1.default.redBright;
function generateWxMp({ weapps, projDir, appId, domain, materials, plugins, isProduction, deployMode, extraData, isMixMode, options, buildTypeList, }) {
    return __awaiter(this, void 0, void 0, function* () {
        const operationLabel = em('Wexin MiniProgram Generated');
        console.time(operationLabel);
        console.log('Generating ' + em('Wexin MiniProgram') + ' to ' + projDir);
        console.log(em('domain') + domain);
        let mainAppData = weapps[0];
        const buildContext = {
            projDir,
            appId,
            isProduction,
            materialLibs: materials,
            isMixMode,
            mainAppData,
            domain,
        };
        const yyptConfig = yield util_3.getYyptConfigInfo(extraData);
        const { appUsedComps, allAppUsedComps } = handleUsedComponents({
            buildContext,
            weapps,
            materials,
        });
        buildContext['miniprogramPlugins'] = (mainAppData.miniprogramPlugins || []).filter((plugin) => allAppUsedComps[plugin.name]);
        yield materials_1.installMaterials(projDir, allAppUsedComps, weapps, buildContext);
        const wxmlDataPrefix = mp_1.getWxmlDataPrefix(!isProduction);
        const { projConfig, appConfig, pageConfigs } = mp_config_1.generateMpConfig(weapps, buildContext);
        if (!mainAppData.mpPkgUrl) {
            const projectFileData = {
                'project.config.json': { content: projConfig },
            };
            console.log('Generating ' + em('project') + ' files');
            yield generateFiles_1.default(projectFileData, templateDir, projDir, buildContext);
        }
        yield generatePkg(mainAppData, path_1.default.join(projDir, '/'), Object.assign(Object.assign({}, buildContext), { rootPath: mainAppData.rootPath }), pageConfigs[0]);
        const projectConfigJsonPath = path_1.default.join(projDir, mainAppData.rootPath || '', 'project.config.json');
        let projectConfigJson = yield fs.readJSON(projectConfigJsonPath);
        const miniprogramRoot = path_1.default.join(projDir, projectConfigJson.miniprogramRoot || '/');
        let appFileData = {};
        if (weapps.find((item) => !item.mpPkgUrl)) {
            yield generateFramework(mainAppData, miniprogramRoot, buildContext);
            if (mainAppData.mpPkgUrl) {
                let appJsPath = path_1.default.join(miniprogramRoot, 'app.js');
                let appJsContent = yield fs.readFile(appJsPath);
                yield generateFiles_1.writeFile(appJsPath, `import { app as wedaApp } from './app/weapps-api'\n${appJsContent}`);
            }
            appFileData = Object.assign(Object.assign({}, appFileData), { 'common/style.js': {}, 'common/utils.wxs': {
                    domain: domain,
                }, 'common/util.js': {
                    isAdminPortal: common_1.buildAsAdminPortalByBuildType(buildTypeList),
                }, 'common/widget.js': {}, 'common/url.js': {}, 'common/weapp-sdk.js': {}, 'common/weapp-page.js': {
                    dataPropNames: wxmlDataPrefix,
                    debug: !buildContext.isProduction,
                }, 'common/weapp-component.js': {}, 'common/merge-renderer.js': {
                    dataPropNames: wxmlDataPrefix,
                    debug: !buildContext.isProduction,
                }, 'common/process.js': {}, 'common/data-patch.js': {} });
        }
        if (mainAppData.mpPkgUrl) {
            if (!projectConfigJson.setting) {
                projectConfigJson.setting = {};
            }
            projectConfigJson.setting = Object.assign(Object.assign({}, projectConfigJson.setting), projConfig.setting);
            yield generateFiles_1.writeFile(projectConfigJsonPath, JSON.stringify(projectConfigJson, undefined, 2));
            let appJsonPath = path_1.default.join(miniprogramRoot, 'app.json');
            let appJson = yield fs.readJson(appJsonPath);
            let subpackages = appJson.subpackages || [];
            for (let item of appConfig.subpackages || []) {
                let find = subpackages.find((config) => config.root === item.root);
                if (find) {
                    find.pages = Array.from(new Set([].concat(find.pages || []).concat(item.pages || [])));
                }
                else {
                    subpackages.push(item);
                }
            }
            appJson.subpackages = subpackages;
            yield generateFiles_1.writeFile(appJsonPath, JSON.stringify(appJson, undefined, 2));
        }
        else {
            appFileData = Object.assign(Object.assign({}, appFileData), { 'common/wx_yypt_report_v2.js': {}, 'app.js': { yyptConfig: yyptConfig }, 'app.json': { content: appConfig }, 'app.wxss': {
                    importStyles: materials.reduce((styles, lib) => {
                        styles = styles.concat((lib.styles || []).map((stylePath) => stylePath && !stylePath.startsWith('/')
                            ? `/${config_1.materialsDirName}/${lib.name}/${stylePath}`
                            : stylePath) || []);
                        return styles;
                    }, []),
                }, 'package.json': {
                    appId,
                    extraDeps: resolveNpmDeps(),
                } });
        }
        console.log('Generating ' + em('miniprogramRoot') + ' files');
        yield generateFiles_1.default(appFileData, templateDir, miniprogramRoot, buildContext);
        if (miniprogramRoot !== path_1.default.join(projDir, '/') &&
            fs.existsSync(path_1.default.join(projDir, config_1.materialsDirName))) {
            console.log('Move ' +
                em(config_1.materialsDirName) +
                ' from' +
                `${projDir} to ${miniprogramRoot}`);
            yield fs.move(path_1.default.join(projDir, config_1.materialsDirName), path_1.default.join(miniprogramRoot, config_1.materialsDirName));
        }
        if (weapps.find((item) => !item.mpPkgUrl)) {
            const datasourceFileData = {
                'datasources/index.js': {},
                'datasources/config.js.tpl': {
                    envID: mainAppData.envId,
                    appID: appId,
                    resourceAppid: !!options.isCrossAccount ? options.resourceAppid : '',
                    isProd: deployMode === types_1.DEPLOY_MODE.UPLOAD,
                },
                'datasources/datasource-profiles.js.tpl': {
                    datasourceProfiles: util_3.JsonToStringWithVariableName(dataSource_1.getDatasourceProfiles(weapps.reduce((datasources, app) => {
                        datasources.push(...(app.datasources || []));
                        return datasources;
                    }, []))),
                },
                'datasources/dataset-profiles.js.tpl': {
                    datasetProfiles: util_3.JsonToStringWithVariableName(dataSource_1.getDatasetProfiles(mainAppData, weapps)),
                },
            };
            console.log('Generating ' + em('datasources') + ' files');
            yield generateFiles_1.default(datasourceFileData, templateDir, miniprogramRoot, buildContext);
        }
        yield Promise.all(weapps.map((app, index) => __awaiter(this, void 0, void 0, function* () {
            if (index == 0) {
                return;
            }
            else {
                const subpackageRootPath = path_1.default.join(miniprogramRoot, app.rootPath || '/');
                let subpackageBuildCtx = Object.assign(Object.assign({}, buildContext), { rootPath: app.rootPath });
                yield generatePkg(app, subpackageRootPath, subpackageBuildCtx, pageConfigs[index]);
                yield generateFramework(app, subpackageRootPath, subpackageBuildCtx);
                let packageJsonPath = path_1.default.join(subpackageRootPath, 'package.json');
                if (!fs.existsSync(packageJsonPath)) {
                    yield generateFiles_1.default({
                        'package.json': {
                            appId,
                            extraDeps: resolveNpmDeps(),
                        },
                    }, templateDir, subpackageRootPath, buildContext);
                }
            }
        })));
        if (fs.existsSync(path_1.default.join(miniprogramRoot, 'package.json'))) {
            yield webpack_1.installDependencies(miniprogramRoot);
        }
        yield handleMpPlugins();
        console.timeEnd(operationLabel);
        cleanProj(weapps, miniprogramRoot);
        cleanMaterils(path_1.default.join(miniprogramRoot, config_1.materialsDirName), allAppUsedComps);
        return { miniprogramRoot };
        function resolveNpmDeps() {
            const deps = weapps.map((app) => app.npmDependencies);
            materials.map((lib) => deps.push(lib.dependencies));
            materials.map((compLb) => {
                if (compLb.isComposite && compLb.compLibCommonResource) {
                    deps.push(compLb.compLibCommonResource.npm || {});
                }
            });
            return deps.reduce((combined, cur) => {
                return Object.assign(Object.assign({}, combined), cur);
            }, {});
        }
        function handleMpPlugins() {
            return __awaiter(this, void 0, void 0, function* () {
                const mpPlugins = (yield plugin_1.getPluginType(miniprogramRoot, plugins)).filter((item) => item.type === 'mp');
                yield plugin_2.runHandleMpPlugin(miniprogramRoot, mpPlugins);
            });
        }
    });
}
exports.generateWxMp = generateWxMp;
function generatePkg(weapp, appRoot, ctx, pageConfigs) {
    return __awaiter(this, void 0, void 0, function* () {
        const wxmlDataPrefix = mp_1.getWxmlDataPrefix(!ctx.isProduction);
        console.log('Generating ' + em(weapp.rootPath ? 'subApp' : 'app') + ' to ' + appRoot);
        if (weapp.mpPkgUrl) {
            console.log(`Removing ${appRoot}`);
            yield generateFiles_1.cleanDir(appRoot, ['materials']);
            yield net_1.downloadZip(weapp.mpPkgUrl, appRoot);
            if (fs.existsSync(path_1.default.join(appRoot, '__MACOSX'))) {
                yield fs.remove(path_1.default.join(appRoot, '__MACOSX'));
            }
        }
        else {
            console.log('Generating ' + em('page') + ' files');
            generateFiles_1.cleanDir(path_1.default.join(appRoot, 'pages'), []);
            yield Promise.all(weapp.pageInstanceList.map((page) => __awaiter(this, void 0, void 0, function* () {
                var _a;
                const rootPath = weapp.rootPath || '';
                const usingComponents = {};
                const wxml = wxml_1.generateWxml(page.componentInstances, `Page ${rootPath ? path_1.default.join(rootPath, 'pages') : ''}/${page.id}`, wxmlDataPrefix, Object.assign(Object.assign({}, ctx), { rootPath, isPage: true }), usingComponents);
                const pageFileName = lodash_1.get(pageConfigs, `${page.id}.pageFileName`, 'index');
                const pageFileData = {
                    [`index.js|${pageFileName}.js`]: {
                        widgetProps: util_2.createWidgetProps(page.componentInstances, ctx),
                        pageUUID: rootPath ? `${rootPath}/${page.id}` : page.id,
                        pageName: page.id,
                        pageSource: page.data.src || '',
                        eventHanlders: util_2.createEventHanlders(page.componentInstances, '$page', ctx),
                        dataBinds: util_2.createDataBinds(page.componentInstances, ctx),
                        debug: !ctx.isProduction,
                        stringifyObj: util_1.inspect,
                        subLevelPath: rootPath ? path_2.default.relative(rootPath, '') + '/' : '',
                    },
                    [`index.json|${pageFileName}.json`]: {
                        usingComponents,
                        extra: getAppendableJson(pageConfigs[page.id]),
                    },
                    [`index.wxml|${pageFileName}.wxml`]: {
                        content: wxml,
                    },
                    [`index.wxss|${pageFileName}.wxss`]: {
                        subWxss: rootPath && !((_a = ctx.mainAppData) === null || _a === void 0 ? void 0 : _a.mpPkgUrl)
                            ? `@import "${path_2.default.relative(`/${rootPath}/pages/${page.id}`, '/lowcode')}/style.wxss";`
                            : '',
                        content: weapps_core_1.toCssText(weapps_core_1.toCssStyle(page.commonStyle, {
                            toRem: false,
                            toRpx: true,
                        }), 'page'),
                        pageWxss: `@import "../../lowcode/${page.id}/style.wxss"`,
                    },
                    'api.js': {},
                };
                yield generateFiles_1.default(pageFileData, templateDir + '/page', path_1.default.join(appRoot, 'pages', page.id), ctx);
            })));
            yield writeLowCodeFiles(weapp, appRoot, ctx);
        }
    });
}
function generateFramework(appData, outDir, ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        let fileData = {};
        if (appData.mpPkgUrl) {
            if (!appData.rootPath) {
                fileData = Object.assign(Object.assign({}, fileData), { 'app/app-global.js': {}, 'app/weapps-api.js': {
                        appId: ctx.appId,
                        subLevelPath: '',
                        subPackageName: '',
                        isBare: true,
                        domain: ctx.domain || '',
                        appConfig: JSON.stringify({}),
                    } });
            }
        }
        else {
            const allCodeModules = [];
            weapps_core_1.loopDealWithFn(appData.pageInstanceList || [], (p) => {
                allCodeModules.push({
                    id: p.id,
                    lowCodes: p.lowCodes || [],
                });
            });
            fileData['app/handlers.js'] = {
                pageModules: allCodeModules.sort(),
            };
            const isMixSubpackage = ctx.isMixMode && appData.rootPath;
            if (!isMixSubpackage) {
                fileData = Object.assign(Object.assign({}, fileData), { 'app/app-global.js': {}, 'app/weapps-api.js': {
                        appId: ctx.appId,
                        domain: ctx.domain || '',
                        subLevelPath: appData.rootPath
                            ? path_1.default.relative(`${appData.rootPath}`, '') + '/'
                            : '',
                        subPackageName: appData.rootPath || '',
                        isBare: false,
                        appConfig: util_3.JsonToStringWithVariableName({
                            id: ctx.appId,
                            pages: appData.pageInstanceList
                                .sort((item) => (item.isHome ? -1 : 1))
                                .map((item) => ({
                                id: item.id,
                            })),
                        }),
                    }, 'app/common.js': {
                        mods: appData.lowCodes
                            .filter((m) => m.type === 'normal-module' && m.name !== '____index____')
                            .map((m) => m.name)
                            .sort(),
                    } });
            }
        }
        console.log('Generate app framework');
        yield generateFiles_1.default(fileData, templateDir, outDir, ctx);
    });
}
function writeLowCodeFiles(appData, outDir, ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        console.log('Writing ' + em('lowcode') + ' files:');
        const lowcodeRootDir = path_1.default.join(outDir, 'lowcode');
        const themeStyle = style_1.generateDefaultTheme(appData);
        if (!(ctx.isMixMode && appData.rootPath)) {
            yield Promise.all(appData.lowCodes
                .filter((mod) => mod.name !== '____index____')
                .map((m) => lowcode_1.writeCode2file(m, lowcodeRootDir, { appDir: outDir }, themeStyle.code)));
        }
        yield Promise.all(weapps_core_1.loopDealWithFn(appData.pageInstanceList, (page) => __awaiter(this, void 0, void 0, function* () {
            var _a;
            style_1.generateDefaultStyle(page);
            yield ((_a = page === null || page === void 0 ? void 0 : page.lowCodes) === null || _a === void 0 ? void 0 : _a.filter((mod) => mod.name !== '____index____').forEach((m) => lowcode_1.writeCode2file(m, lowcodeRootDir, { pageId: page.id, appDir: outDir }, themeStyle.code, ctx)));
        })));
    });
}
exports.writeLowCodeFiles = writeLowCodeFiles;
function getAppendableJson(obj) {
    if (obj && Object.keys(obj).length > 0) {
        const str = JSON.stringify(obj);
        return ',\n' + str.substr(1, str.length - 2);
    }
    return '';
}
function handleUsedComponents({ buildContext, weapps, materials, }) {
    const appUsedComps = weapps.map((app) => {
        var _a;
        const usedComps = {};
        (_a = app.pageInstanceList) === null || _a === void 0 ? void 0 : _a.forEach((p) => wxml_1.getUsedComponents(p.componentInstances, usedComps));
        return {
            rootPath: app.rootPath || '',
            usedComps,
        };
    });
    let allAppUsedComps = appUsedComps.reduce((comps, item) => {
        Object.keys(item.usedComps).forEach((libName) => {
            comps[libName] = new Set([
                ...Array.from(item.usedComps[libName]),
                ...Array.from(comps[libName] || []),
            ]);
        });
        return comps;
    }, {});
    const compositedLibs = materials.filter((lib) => lib.isComposite);
    allAppUsedComps = materials_1.extractUsedCompsRecursively(allAppUsedComps, [], compositedLibs);
    if (buildContext.isMixMode) {
        appUsedComps.forEach((item) => {
            const appCompositedLibs = materials.filter((lib) => lib.isComposite);
            item.usedComps = materials_1.extractUsedCompsRecursively(item.usedComps, [], appCompositedLibs);
        });
    }
    return {
        appUsedComps,
        allAppUsedComps,
    };
}
exports.handleUsedComponents = handleUsedComponents;
function cleanProj(weapps, miniprogramRoot) {
    return __awaiter(this, void 0, void 0, function* () {
        weapps.map((pkg) => cleanPkg(pkg, miniprogramRoot));
    });
}
function cleanPkg(pkg, miniprogramRoot) {
    return __awaiter(this, void 0, void 0, function* () {
        if (pkg.mpPkgUrl) {
            return;
        }
        const pkgDir = [miniprogramRoot, pkg.rootPath]
            .filter((p) => !!p)
            .join(path_1.default.sep);
        const pagesDir = path_1.default.join(pkgDir, 'pages');
        const lowcodesDir = path_1.default.join(pkgDir, 'lowcode');
        const existedPages = yield fs.readdir(pagesDir);
        const pages = pkg.pageInstanceList.map((p) => p.id);
        existedPages.forEach((pageName) => {
            var _a, _b, _c, _d;
            const pageDir = path_1.default.join(pagesDir, pageName);
            const lowcodeDir = path_1.default.join(lowcodesDir, pageName);
            if (pages.indexOf(pageName) < 0) {
                generateFiles_1.removeFile(pageDir);
                generateFiles_1.removeFile(lowcodeDir);
            }
            else {
                const handlersDir = path_1.default.join(lowcodeDir, 'handler');
                const handlers = ((_d = (_c = (_b = (_a = pkg === null || pkg === void 0 ? void 0 : pkg.pageInstanceList) === null || _a === void 0 ? void 0 : _a.find((p) => p.id === pageName)) === null || _b === void 0 ? void 0 : _b.lowCodes) === null || _c === void 0 ? void 0 : _c.filter((m) => m.type === 'handler-fn')) === null || _d === void 0 ? void 0 : _d.map((m) => m.name + '.js')) || [];
                generateFiles_1.cleanDir(handlersDir, handlers);
            }
        });
        const commonModules = pkg.lowCodes
            .filter((m) => m.type === 'normal-module')
            .map((m) => m.name + '.js');
        const commonDir = path_1.default.join(lowcodesDir, 'common');
        generateFiles_1.cleanDir(commonDir, commonModules);
    });
}
function cleanMaterils(materialsDir, usedComps) {
    if (fs.existsSync(materialsDir)) {
        fs.readdirSync(materialsDir)
            .filter(junk.not)
            .map((libName) => {
            const libDir = path_1.default.join(materialsDir, libName);
            if (fs.existsSync(path_1.default.join(libDir, 'meta.json'))) {
                try {
                    generateFiles_1.removeFile(path_1.default.join(libDir, 'meta.json'));
                }
                catch (e) { }
                return;
            }
            if (!usedComps[libName]) {
                generateFiles_1.removeFile(libDir);
                return;
            }
            generateFiles_1.cleanDir(libDir, [...Array.from(usedComps[libName]), 'libCommonRes']);
        });
    }
}
