"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeCode2file = void 0;
const path_1 = __importDefault(require("path"));
const weapps_core_1 = require("../../weapps-core");
const style_1 = require("../util/style");
const generateFiles_1 = require("../util/generateFiles");
function writeCode2file(mod, lowcodeRootDir, opts = {}, themeCode, ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        const { pageId = 'global', appDir, comp } = opts;
        const file = path_1.default.join(lowcodeRootDir, weapps_core_1.getCodeModuleFilePath(pageId, mod, { style: '.wxss' }));
        let code = mod.code;
        if (mod.type !== 'style' && mod.type !== 'theme') {
            if (appDir) {
                const baseDir = path_1.default
                    .relative(path_1.default.dirname(file), appDir)
                    .replace(/\\/g, '/');
                const relativeRoot = (ctx === null || ctx === void 0 ? void 0 : ctx.isMixMode) && ctx.rootPath
                    ? path_1.default.relative(ctx.rootPath, '') + '/'
                    : '';
                let weappsApiPrefix = `import { app, process } from '${relativeRoot}${baseDir}/app/weapps-api';\nconst $app = app;`;
                if (pageId !== 'global') {
                    weappsApiPrefix += `\nimport { $page } from '${baseDir}/pages/${pageId}/api'`;
                }
                code = weappsApiPrefix + '\n' + code;
            }
            else {
                code = `import process from '${mod.type === 'handler-fn' ? '../' : ''}../../../../common/process'\nimport app from '${mod.type === 'handler-fn' ? '../' : ''}../../../../common/weapp-sdk';\nconst $app = app;\n${code.replace(/\$comp/g, weapps_core_1.COMPONENT_API_PREFIX)}`;
            }
        }
        else {
            code = `${themeCode ? themeCode : ''}\n${code}`;
            if (comp) {
                code = `.${weapps_core_1.getCompositedComponentClass(comp)} {\n${code}\n}`;
            }
            try {
                code = yield style_1.processLessToRpx(`${code}`);
            }
            catch (e) {
                console.error('processLess Error', e);
            }
        }
        yield generateFiles_1.writeFile(file, code);
    });
}
exports.writeCode2file = writeCode2file;
