/// <reference types="node" />
export * as generator from './generator/core/index';
import { PathLike } from 'fs-extra';
import { Plugin, PluginServiceApi } from '@cloudbase/framework-core';
import { BuildType, GenerateMpType } from './builder/types/common';
import { IMaterialItem, IPlugin } from './weapps-core';
import { DEPLOY_MODE, RUNTIME } from './types';
export declare const DIST_PATH = "./dist";
export declare const PERSISTENT_DEPENDIENCES_MAP: {};
declare const DEFAULT_INPUTS: {
    _inputFile: string;
    debug: boolean;
    runtime: RUNTIME;
    appId: string;
    buildTypeList: BuildType[];
    generateMpType: GenerateMpType;
    generateMpPath: string;
    subAppSerializeDataList: never[];
    dependencies: never[];
    plugins: never[];
    operationService: {};
    extraData: {
        isComposite: boolean;
        compProps: {};
    };
    mpAppId: string;
    mpDeployPrivateKey: string;
    deployOptions: {
        mode: string;
    };
    calsVersion: string;
    ignoreInstall: boolean;
};
export interface IFrameworkPluginLowCodeInputs {
    _inputFile?: string;
    debug?: boolean;
    runtime?: RUNTIME;
    appId: string;
    mainAppSerializeData: any;
    subAppSerializeDataList?: any[];
    dependencies?: IMaterialItem[];
    buildTypeList?: BuildType[];
    generateMpType?: GenerateMpType;
    mpAppId?: string;
    generateMpPath?: string;
    plugins?: IPlugin[];
    mpDeployPrivateKey?: string;
    deployOptions: {
        mode: DEPLOY_MODE;
        version?: string;
        description?: string;
        mpAppId?: string;
        mpDeployPrivateKey?: string;
        targetMpAppId?: string;
        publicPath?: string;
    };
    credential?: {
        secretId: string;
        secretKey: string;
        token?: string;
    };
    storage?: {
        bucket: string;
        region: string;
    };
    extraData?: {
        operationService?: Object;
        isComposite: boolean;
        compProps: any;
    };
    calsVersion?: string;
    ignoreInstall?: boolean;
}
export declare type ResolvedInputs = IFrameworkPluginLowCodeInputs & typeof DEFAULT_INPUTS;
declare class LowCodePlugin extends Plugin {
    name: string;
    api: PluginServiceApi;
    inputs: IFrameworkPluginLowCodeInputs;
    protected _resolvedInputs: ResolvedInputs;
    protected _appPath: string;
    protected _authPlugin: any;
    protected _miniprogramePlugin: any;
    protected _webPlugin: any;
    protected _functionPlugin: any;
    protected _productBasePath?: string;
    protected _timeMap: {};
    protected _logFilePath?: PathLike;
    protected _domain: any[];
    protected _website: any;
    protected _rules: any;
    protected _skipInstallExt: any;
    protected _gatewayConfig?: {
        Domain: string;
        Path: string;
    };
    constructor(name: string, api: PluginServiceApi, inputs: IFrameworkPluginLowCodeInputs);
    _initDir(): void;
    _subPluginConstructor(resolveInputs: ResolvedInputs): void;
    _getWebRootPath(): string;
    _time(label: any): void;
    _timeEnd(label: any): number | undefined;
    init(): Promise<void>;
    run(): Promise<void>;
    remove(): Promise<void>;
    genCode(): Promise<void>;
    build(): Promise<string>;
    compile(): Promise<any>;
    deploy(): Promise<void>;
    mpAnalyzeAndCleanCode(distDir: string): Promise<void>;
    _getHostingInfoTask(envId: any, loose?: boolean): Promise<any[]>;
    _getHostingInfo(envId: any, loose?: boolean): any;
    _checkIsVersion(version: any): boolean;
    _handleCIProduct(): Promise<void>;
    _postPersistentDependiences(): Promise<void>;
    _debugInfo(): Promise<void>;
    _zipDir(src: any, dist: any): Promise<unknown>;
    _postProcessAdminPortal(): Promise<void>;
    _postProcessWebsiteConfig(): Promise<void>;
    _checkBroswerHisroty(): boolean;
}
export declare const plugin: typeof LowCodePlugin;
//# sourceMappingURL=index.d.ts.map