"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDatasetProfiles = exports.getDataVarProfiles = exports.getDatasourceProfiles = exports.standardizeDataSource = exports.lowercaseKey = exports.mergeDependencies = exports.getDatasourceDatabaseName = exports.getAppDatasourceResourceName = exports.getDatasourceResourceName = exports.DATABASE_TYPE = exports.EXTERNAL_FUNCTION_TYPE = exports.CLOUD_FUNCTION_TYPE = void 0;
const merge_package_json_1 = __importDefault(require("merge-package-json"));
const types_1 = require("../types");
const expression_1 = require("@cloudbase/cals/lib/parser/expression");
const util_1 = require("../builder/mp/util");
const config_1 = require("../builder/config");
exports.CLOUD_FUNCTION_TYPE = 'cloud-function';
exports.EXTERNAL_FUNCTION_TYPE = 'http';
exports.DATABASE_TYPE = 'database';
function getDatasourceResourceName(datasrouce, mode) {
    let suffix = mode === types_1.DEPLOY_MODE.PREVIEW ? `-preview` : '';
    return `lcap-${datasrouce.id}-${datasrouce.name}${suffix}`;
}
exports.getDatasourceResourceName = getDatasourceResourceName;
function getAppDatasourceResourceName(appId, datasrouce, mode) {
    let suffix = mode === types_1.DEPLOY_MODE.PREVIEW ? `-preview` : '';
    return `lcap-${datasrouce.id}-${datasrouce.name}-${appId}${suffix}`;
}
exports.getAppDatasourceResourceName = getAppDatasourceResourceName;
function getDatasourceDatabaseName(dsConfig, mode) {
    return dsConfig.type === exports.DATABASE_TYPE
        ? getDatasourceResourceName(dsConfig, mode)
        : null;
}
exports.getDatasourceDatabaseName = getDatasourceDatabaseName;
function mergeDependencies(...pkgs) {
    let result = { dependencies: {} };
    result = pkgs.reduce((acc, pkg) => {
        var _a;
        const formated = getDependencies(pkg);
        for (let key in formated.dependencies) {
            formated.dependencies[key] =
                formated.dependencies[key] === '*' && ((_a = acc === null || acc === void 0 ? void 0 : acc.dependencies) === null || _a === void 0 ? void 0 : _a[key]) === '*'
                    ? 'latest'
                    : formated.dependencies[key];
        }
        const merged = merge_package_json_1.default(formated, acc);
        return JSON.parse(merged);
    }, result);
    return result.dependencies;
}
exports.mergeDependencies = mergeDependencies;
function getDependencies(pkg) {
    if (pkg.dependencies && typeof pkg.dependencies === 'object') {
        return {
            dependencies: pkg.dependencies,
        };
    }
    return {
        dependencies: pkg,
    };
}
function lowercaseKey(obj, deep) {
    if (!obj || typeof obj !== 'object')
        return obj;
    if (Array.isArray(obj))
        return obj.map((item) => lowercaseKey(item, deep));
    const result = {};
    return Object.keys(obj).reduce((acc, key) => {
        const smallCase = key.charAt(0).toLowerCase() + key.slice(1);
        const val = obj[key];
        acc[smallCase] = deep ? lowercaseKey(val, deep) : val;
        return acc;
    }, result);
}
exports.lowercaseKey = lowercaseKey;
function standardizeDataSource(ds) {
    const transformed = lowercaseKey(ds, true);
    try {
        const { schema } = ds;
        if (typeof transformed.schema === 'string') {
            transformed.schema = JSON.parse(transformed.schema);
        }
        if (!transformed.schema || typeof transformed.schema !== 'object') {
            transformed.schema = {};
        }
    }
    catch (error) {
        transformed.schema = {};
    }
    try {
        const { methods } = ds;
        if (typeof transformed.methods === 'string') {
            transformed.methods = JSON.parse(transformed.methods);
        }
        if (!Array.isArray(transformed.methods)) {
            transformed.methods = [];
        }
    }
    catch (error) {
        transformed.methods = [];
    }
    return transformed;
}
exports.standardizeDataSource = standardizeDataSource;
function getDatasourceProfiles(datasources) {
    return ((datasources === null || datasources === void 0 ? void 0 : datasources.map((item) => {
        const ds = standardizeDataSource(item);
        return {
            id: ds.id,
            title: ds.title,
            name: ds.name,
            type: ds.type,
            childDataSourceNames: ds.childDataSourceNames,
            schema: ds.schema,
            methods: ds.methods &&
                ds.methods.map((method) => {
                    return {
                        name: method.name,
                        type: method.type,
                    };
                }),
        };
    })) || []);
}
exports.getDatasourceProfiles = getDatasourceProfiles;
function getDataVarProfiles(appData) {
    const result = {
        $global: (appData.vars && appData.vars.data) || [],
    };
    appData.pageInstanceList.forEach((pageInstance) => {
        let p = pageInstance;
        result[p.id] = (p.vars && p.vars.data) || [];
    });
    return result;
}
exports.getDataVarProfiles = getDataVarProfiles;
function _generateDynamicDataset(dataset) {
    var _a;
    let { state } = dataset;
    if (state) {
        for (let key in state) {
            let config = state[key];
            if (config.varType === 'datasource' && ((_a = config.initMethod) === null || _a === void 0 ? void 0 : _a.params)) {
                let params = config.initMethod.params;
                if (params) {
                    let processed = {};
                    for (let paramKey in params) {
                        let bind = params[paramKey];
                        if (!bind.type || bind.type === expression_1.PropBindType.static) {
                            let value = bind.value;
                            processed[paramKey] = `${config_1.REPLACE_SIGN}(app, $page) => (${typeof value === 'string' ? `'${value}'` : JSON.stringify(value)})${config_1.REPLACE_SIGN}`;
                        }
                        else {
                            let jsExp = util_1.generateDataBind(bind);
                            processed[paramKey] = `${config_1.REPLACE_SIGN}(app, $page) =>  (\n${jsExp}\n)${config_1.REPLACE_SIGN}`;
                        }
                    }
                    config.initMethod.params = processed;
                }
            }
        }
    }
    return dataset;
}
function getDatasetProfiles(mainAppData, apps) {
    const result = {};
    if (mainAppData.dataset) {
        result['$global'] = _generateDynamicDataset(mainAppData.dataset);
    }
    (apps || []).forEach((app) => {
        var _a;
        (_a = app.pageInstanceList) === null || _a === void 0 ? void 0 : _a.forEach((pageInstance) => {
            let p = pageInstance;
            if (p.dataset) {
                result[app.rootPath ? `${app.rootPath}/${p.id}` : p.id] =
                    _generateDynamicDataset(p.dataset);
            }
        });
    });
    return result;
}
exports.getDatasetProfiles = getDatasetProfiles;
