// Import Libs and Handlers
import * as React from 'react'
import { observable } from 'mobx'
import { AppRender } from 'handlers/render'
import { initLifeCycle, pageLifeCycleMount } from 'handlers/lifecycle'
import { createComputed } from 'utils'
import AppLifeCycle from 'lowcode/lifecycle'
import { createDataset, createStateDataSourceVar, generateParamsParser, EXTRA_API } from '../../datasources'
import { $$_<%= pageName %> as handler } from '../../app/handlers'
import { app as mainApp } from 'app/global-api' // 取主包app
import { app, $page } from '../../app/global-api' // 取对应子包app
import { createWidgets, retryDataBinds, resolveComponentProps, checkAuth, reportTime } from 'handlers/utils'
import { useScrollTop } from 'handlers/hooks'
import { get } from "lodash";
import './index.less'
<% if(pageSource){ %>
import { loadMicroApp } from 'qiankun';
const PageLifeCycle = {};
const initPageState = {};
const computed = {}
<% } else { %>
import PageLifeCycle from '../../lowcode/<%= pageName %>/lifecycle'
import initPageState from '../../lowcode/<%= pageName %>/state'
import computed from '../../lowcode/<%= pageName %>/computed'
function loadMicroApp(){
  return new Proxy({}, {
    get(obj, prop) {
      return (...args) => {};
    },
  })
}
<% } %>

let ReactDOMServer;

if(process.env.SSR) {
  ReactDOMServer = require('react-dom/server');
}

// Import dependencies entry
<%= entryImports %>

// Import Components
<%= componentImports %>

// Import Plugins
<%= pluginImports %>

// Import Actions
<%= actionImports %>

// Actions

// Plugin
const pluginInstances = <%= pluginInstances %>;
const virtualFields = <%= virtualFields %>;
const componentSchema = <%= componentSchema %>;
const pageListenerInstances = <%= pageListenerInstances %>;
const widgetsContext = <%= widgets %>;
const dataBinds = <%= dataBinds %>;

AppLifeCycle.beforeCustomLaunch = (query)=>{
  EXTRA_API.setParams('$global', query || {})
  createStateDataSourceVar('$global',generateParamsParser({app}))
};
PageLifeCycle.beforePageCustomLaunch = (query) => {
  EXTRA_API.setParams('<%= pageName %>', query || {})
  createStateDataSourceVar('<%= pageName %>',generateParamsParser({app, $page}))
};
// lifecycle
initLifeCycle({
  ...AppLifeCycle,
  ...PageLifeCycle
}, app, mainApp)


// Init
export default function App() {
  // 检查权限
  const [weDaHasLogin, setWeDaHasLogin] = React.useState(false);
  const context = React.useRef(observable({})).current;
  const containerRef = React.useRef(null);
  const microApp = React.useRef(null);
  const pureSrc = '<%= pageSource %>';
  const isPure = !!pureSrc;

  /**
   * 更新数据容器的上下文的方法
   * 会传递到事件emit的地方，将从组件获取到的数据data赋值给上下文
   * 当组件卸载时，传过来的data为undefined即可
   * {
   *   id1: [{...}],
   *   id2: {...},
   *   id3: undefined,
   *   id4: null,
   *   ...
   * }
   * @param id
   * @param data
   */
  const updateContext = (id, data) => {
    if (id) {
      context[id] = { data };
    }
  }

  React.useEffect(() => {
    checkAuth(app, app.id, '<%= pageName %>').then((checkAuthResult) => {
      setWeDaHasLogin(checkAuthResult)
      reportTime('FIRST_PAGE_CONTENT_RENDERED', undefined, true)
    });

    Object.assign($page, {
      id: '<%= pageName %>',
      state: observable(initPageState),
      computed: createComputed(computed),
      handler,
    });

    let dataset = createDataset('<%= pageName %>', { app, $page });
    $page.dataset = dataset;
    $page.state.dataset = dataset;
    $page.setState = (userSetState) => {
      Object.keys(userSetState).forEach((keyPath) => {
        app.utils.set($page.dataset.state, keyPath, userSetState[keyPath]);
      });
    };
    return () => {
      if (microApp.current) {
        microApp.current.unmount();
      }
    };
  }, []);

  React.useEffect(() => {
    $page.widgets = createWidgets(widgetsContext, dataBinds, {}, context);
    // widgets 内的 dataBinds 可能需要关联 widgets，需要重新执行 dataBinds
    retryDataBinds();
  }, [context]);

  // Web 环境页面级别生命周期
  if (!process.env.isMiniprogram) {
    React.useEffect(() => {
      document.title = "<%= title %>"
    }, [])
    pageLifeCycleMount(React.useEffect, PageLifeCycle, app)
  }

  React.useEffect(() => {
    if (isPure) {
      if (weDaHasLogin) {
        if (!microApp.current) {
          microApp.current = loadMicroApp({
            name: '<%= pageName %>',
            entry: pureSrc,
            container: containerRef.current,
          });
        } else {
          microApp.current.mount?.();
        }
      } else if (microApp.current) {
        microApp.current.unmount();
      }
    }
  }, [weDaHasLogin]);

  useScrollTop();

  return (
    <div className="weapps-page" ref={containerRef}>
      <style
        type="text/css"
        dangerouslySetInnerHTML={{
          __html: `<%= pageStyleText %>`,
        }}
      />
      {weDaHasLogin && !isPure && (
        <AppRender pageListenerInstances={pageListenerInstances}
          virtualFields={virtualFields}
          componentSchema={componentSchema}
          context={context}
          updateContext={updateContext}
        />
      )}
    </div>
  );
}

export function renderToString() {
  return ReactDOMServer.renderToString(<App />);
}
